# significance testing with classical methods (Anova, Tukey HSD, Wilcox) relies on 
# various assumptions. Instead of trying to fit them all, we can use bootstrapping, 
# which is more flexible and equally robust. 

# we want to use bias-corrected accelerated (BCa) confidence intervals, 
# which are much trickier to calculate than basic and percentile intervals, 
# but more robust

# the workflow for this consists of four steps: 
# 1. nest the data
# 2. calculate bootstrapped samples for the focal statistic
# 3. calculate confidence intervals from bootstrappped samples
# 4. unnest the data

# let's set up the functions for these steps


# functions ---------------------------------------------------------------


# the first step when using boot::boot (the core function in the 
# package, which performs the resampling) is to provide a user-defined function 
# that calculates the statistic of interest. In this case I define a function 
# boot_median that calculates the median of a dataset. 
# In its simplest form, this median unction has two parameters, d and i,
# where d is the data that will be used to sample from,
# and i is an index parameter used by the internals of boot::boot 
# to take a random sample (with replacement) from d.

# define the bootstrap function
boot_median <- function(d, i) {
  median(d[i])
}


# calculate the bootstrapped samples and ci's. 
# the function takes a nested data frame and returns a nested dfr. 


sample_boot <- function(nested_dfr, # the nested dataframe
                      nr_rep = 1e4, # number of replicates
                      parallel_cpu = "no", # set parallel, default to no
                      nr_cpu = 1, # number of cores
                      boot_type = "bca" # bootstrapp type, default BCa interval
                      ) {
  
  boot_results <- nested_dfr %>% 
    # bootstrapp recorded data
    mutate(booted_diff = map(.x = data,
                             ~ boot::boot(data = .x$difference, # sampled data
                                          statistic = boot_median, # function
                                          R = nr_rep, 
                                          stype = "i", 
                                          parallel = parallel_cpu, 
                                          ncpus = nr_cpu))) %>% 
    # calculate ci's
    mutate(booted_ci = map(.x = booted_diff, 
                             ~ boot::boot.ci(.x,
                                             conf = 0.95, # 95 CI's
                                             type = boot_type)))  
  # return nested dfr
  boot_results
  
}


# unnest the bootstrapped samples to get the statistic and ci
# the function takes a booted nested dfr and returns a dfr

unnest_boot <- function(nested_boot_dfr, # booted nested dataframe
                        boot_type = "bca" # bootstrapp type, default BCa interval
                        ) {
  
  unnested_results <- nested_boot_dfr %>%
    # Add columns
    mutate(statistic = map(.x = booted_ci, # booted ci object
                           ~ .x$t0), # point estimate
           lower_ci = map(.x = booted_ci,
                          ~ .x$bca[[4]]), # value of lower 2.5% limit
           upper_ci = map(.x = booted_ci,
                          ~ .x$bca[[5]])) %>% # value of  upper 97.5% limit
    # drop list-columns
    select(-data, -booted_diff, -booted_ci) %>%
    # unnest the dataframe
    unnest(cols = c(statistic, lower_ci, upper_ci))
  
  # return unnested dfr
  unnested_results
  
}

