# load packages in memory
library(tidyverse)
library(here)
library(boot)



# functions ---------------------------------------------------------------


# the workflow for bootstrapping consists of four steps: 
# 1. nest the data
# 2. calculate bootstrapped samples for the focal statistic
# 3. calculate confidence intervals from bootstrappped samples
# 4. unnest the data

# the functions are defined in "boot_functions.R".

source("boot_functions.R")



# load data ---------------------------------------------------------------

bike_predicted <- read_csv(here("data/bike_predicted.csv"))

# subset data to model 1 and calculate differences
bike_predicted <- bike_predicted %>% 
  select(Strecke:recorded, predicted, type) %>% 
  drop_na(recorded, predicted) %>% 
  mutate(difference = recorded - predicted) %>% 
  select(Strecke:Jahr,type, difference)

# nest the data
bike_nested <- bike_predicted %>% 
  group_by(Monat, type) %>%
  nest()
 
# take a look
# lapply(bike_nested$data, head, n = 2)



# bootstrapping -----------------------------------------------------------

# get bootstrapped samples and 95% Confidence Intervals using BCa intervals
# with increasing replicates, the computational power required increases
# if you have more than one cpu core, use them all by setting parallel computation and number of cpu's
bike_booted <- sample_boot(nested_dfr = bike_nested, 
                           nr_rep = 1e5, 
                           parallel = "multicore", 
                           nr_cpu = 8)



# unnest the data
bike_booted <- unnest_boot(nested_boot_dfr = bike_booted)






# bootstrapped summaries --------------------------------------------------

# let's get a summary table
# compare 2019 to 2020 and don't split it into each month, take bootstrapped
# samples again and calculate booted differences. Same procedure as before. 

# nest the data per year and type
bike_nested_smr <- bike_predicted %>% 
  group_by(Jahr, type) %>%
  nest()

# bootstrapping

# unnest
bike_booted_smr <- sample_boot(nested_dfr = bike_nested_smr, 
                           nr_rep = 1e5, 
                           parallel = "multicore", 
                           nr_cpu = 8)



# unnest the data
bike_booted_smr <- unnest_boot(nested_boot_dfr = bike_booted_smr)



# percentage differences --------------------------------------------------

# these results show the changes in median response of cycling frequency
# let's transform these natural values to percentages, to get a better overview

bike_predicted <- read_csv(here("data/bike_predicted.csv"))

# subset data to model 1 and calculate percentage differences
bike_predicted <- bike_predicted %>% 
  select(Strecke:recorded, predicted, type) %>% 
  drop_na(recorded, predicted) %>% 
  mutate(difference = ((recorded - predicted)/predicted)*100) %>% 
  select(Strecke:Jahr,type, difference)

# nest the data
bike_nested <- bike_predicted %>% 
  group_by(Monat, type) %>%
  nest()


# bootstrapping

# get bootstrapped samples and 95% Confidence Intervals using BCa intervals
# with increasing replicates, the computational power required increases
# if you have more than one cpu core, use them all by setting parallel computation and number of cpu's
bike_booted_perc <- sample_boot(nested_dfr = bike_nested, 
                           nr_rep = 1e5, 
                           parallel = "multicore", 
                           nr_cpu = 8)



# unnest the data
bike_booted_perc <- unnest_boot(nested_boot_dfr = bike_booted_perc)



# let's get a summary table
# compare 2019 to 2020 and don't split it into each month, take bootstrapped
# samples again and calculate booted differences. Same procedure as before. 

# nest the data per year and type
bike_nested_smr <- bike_predicted %>% 
  group_by(Jahr, type) %>%
  nest()

# bootstrapping

# unnest
bike_booted_smr_perc <- sample_boot(nested_dfr = bike_nested_smr, 
                               nr_rep = 1e5, 
                               parallel = "multicore", 
                               nr_cpu = 8)



# unnest the data
bike_booted_smr_perc <- unnest_boot(nested_boot_dfr = bike_booted_smr_perc)





# save data ---------------------------------------------------------------


# for median differences per month and type
write_csv(bike_booted, here("results/median_differences.csv"))

# for median differences for year and type
write_csv(bike_booted_smr, here("results/median_differences_smr.csv"))

# for median differences per month and type
write_csv(bike_booted_perc, here("results/median_percentage_differences.csv"))

# for median differences for year and type
write_csv(bike_booted_smr_perc, here("results/median_percentage_differences_smr.csv"))






