# load libraries
library(readxl)
library(here)
library(tidyverse)


# strava data ---------------------------------------------------------------

# load data
daten <- read_csv(here("data/strava_raw.csv"))

# clean data
daten <- daten %>% 
  filter(Jahr > 2012) %>% 
  unite("ID", Jahr, Monat, Region, Segment, remove = FALSE)

# subset to bikers
bike <- daten %>% 
  filter(Aktivitaet == "Radfahren") %>% 
  select(-Aktivitaet) 

# count observations per ID
id_count <- count(bike, ID, name = "N")

# merge to bikes to get columns
bike <- bike %>% 
  select(-Name) %>% 
  right_join(id_count) %>% 
  distinct() %>% 
  arrange(ID)




# climate data ------------------------------------------------------------

# set up list for historical data
klima.hist <- list()

# loop through wetterstationen
for (i in unique(bike$Wetterstation)) {
  if (nchar(i) == 2) {
    ID <- paste("000", i, sep = "")
  }
  if (nchar(i) == 3) {
    ID <- paste("00", i, sep = "")
  }
  if (nchar(i) == 4) {
    ID <- paste("0", i, sep = "")
  }
  if (nchar(i) == 5) {
    ID <- i
  }
  
  # read in climate data
  dat <- read_csv2(paste(
    here("data/Wetterdaten_DWD_Anne/hist/produkt_klima_monat_"), 
    formatC(ID, width = 5, format = "d", flag = "0"),
    ".txt", sep="")
    )
  
  # clean climate data
  dat <-  dat %>% 
    # get year and month
    mutate(Jahr = as.numeric(substring(dat$MESS_DATUM_BEGINN, 1, 4)),
           Monat = as.numeric(substring(dat$MESS_DATUM_BEGINN, 5, 6))) %>% 
    # assign missing values
    mutate(across(where(is.numeric), na_if, -999), 
           across(where(is.character), na_if, "-999")) 
 

  klima.hist[[as.character(i)]] <- dat
}

# clean up
rm(dat, ID)

# same for recent data
klima.recent <- list()

for (i in unique(bike$Wetterstation)){
  if (nchar(i) == 2) {
    ID <- paste("000", i, sep = "")
  }
  if (nchar(i) == 3) {
    ID <- paste("00", i, sep = "")
  }
  if (nchar(i) == 4) {
    ID <- paste("0", i, sep = "")
  }
  if (nchar(i) == 5) {
    ID <- i
  }
  
  # read in climate data
  dat <- read_csv2(paste(
    here("data/Wetterdaten_DWD_Anne/akt/produkt_klima_monat_"), 
    formatC(ID, width = 5, format = "d", flag = "0"),
    ".txt", sep="")
  )
  
  # clean climate data
  dat <-  dat %>% 
    # get year and month
    mutate(Jahr = as.numeric(substring(dat$MESS_DATUM_BEGINN, 1, 4)),
           Monat = as.numeric(substring(dat$MESS_DATUM_BEGINN, 5, 6))) %>% 
    # assign missing values
    mutate(across(where(is.numeric), na_if, -999), 
           across(where(is.character), na_if, "-999")) 
  
  
  klima.recent[[as.character(i)]] <- dat
}

# clean up
rm(dat, ID)



# calculate climate -------------------------------------------------------

# get climate per wetterstation


# set up columns
bike$Temperatur <- NA
bike$Niederschlag <- NA
bike$Sonnenscheindauer <- NA

# loop through rows
# this might take a while, so I'm going to set up a progress bar
pb <- progress::progress_bar$new(total = nrow(bike))

for (i in 1:nrow(bike)) {
  temp <- klima.recent[[as.character(bike$Wetterstation[i])]]
  temp <- temp[!duplicated(paste(temp$Jahr, temp$Monat)), ]
  if (sum(temp$Jahr == bike$Jahr[i] & temp$Monat == bike$Monat[i]) > 0) {
    bike$Temperatur[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_TT"]))
    bike$Niederschlag[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_RR"]))
    bike$Sonnenscheindauer[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_SD_S"]))
  }
  rm(temp)
  temp <- klima.hist[[as.character(bike$Wetterstation[i])]]
  temp <- temp[!duplicated(paste(temp$Jahr, temp$Monat)), ]
  if (sum(temp$Jahr == bike$Jahr[i] & temp$Monat == bike$Monat[i]) > 0) {
    bike$Temperatur[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_TT"]))
    bike$Niederschlag[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_RR"]))
    bike$Sonnenscheindauer[i]	<-
      as.numeric(as.character(temp[temp$Jahr == bike$Jahr[i] &
                                     temp$Monat == bike$Monat[i], "MO_SD_S"]))
  }
  pb$tick()
  rm(temp, i)
}



# clean data --------------------------------------------------------------


# remove juli and august
bike <-  bike[!c(bike$Monat > 6 & bike$Jahr == 2020), ]

# save data to csv
write_csv(bike, file = here("data/bike_raw.csv"))

