# load packages in memory
library(tidyverse)
library(here)
library(lubridate)
library(flextable)
library(officer)
library(patchwork)


# this script aims to generate plots and tables for the bootstrapped differences
# I, personally, would report the plots in percentage difference, for a better overview, and add some tables
# with raw differences in the supplemental data

# set up locale to english for date plotting
Sys.setlocale("LC_TIME", "en_US.utf8")

# read in data ------------------------------------------------------------

bike_booted <- read_csv(here("results/median_differences.csv"))

bike_booted_smr <- read_csv(here("results/median_differences_smr.csv"))

bike_booted_perc <- read_csv(here("results/median_percentage_differences.csv"))

bike_booted_smr_perc <- read_csv(here("results/median_percentage_differences_smr.csv"))


# plot --------------------------------------------------------------------


# summary plot
summary_plot <- bike_booted_smr_perc %>% 
  add_column(x_pos = c(2.5, 1, 3, 1.5)) %>% 
  ggplot(aes(x = x_pos, y = statistic)) +
  geom_col(aes(fill = type), alpha = 0.8) +
  geom_errorbar(aes(ymin = lower_ci, ymax = upper_ci),
                width = 0.2, colour = "grey40") +
  geom_hline(yintercept = 0, colour = "grey20") +
  scale_x_continuous(breaks = c(1.25, 2.75), labels = c("2019", "2020")) +
  scale_fill_manual(values = c("chartreuse3", "orange")) +
  scale_y_continuous(labels = function(x) paste0(x, "%")) +
  labs(y = "Median difference in user frequency", x = NULL, 
       fill = "Sample site") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.text.x = element_text(size = 11), 
        legend.position = c(0.15,0.85), 
        legend.background = element_rect(colour = "grey70"))



# per month and type
# generate date format for plotting
bike_booted_date <- bike_booted_perc %>% 
  ungroup() %>% 
  select(month = Monat, everything()) %>% 
  mutate(year = if_else(month %in% c(3:6), 2020, 2019),
         recorded_date = make_date(year = year, month = month), 
         .before = month) %>% 
  select(-month, -year)

# split per type
bike_booted_rur <- bike_booted_date %>% 
  filter(type == "rural") %>% 
  select(-type)

bike_booted_urb <- bike_booted_date %>% 
  filter(type == "urban") %>% 
  select(-type)

# plot per type
rural_plot <- 
  ggplot(bike_booted_rur, aes(x = recorded_date, y = statistic)) +
  geom_hline(yintercept = 0, colour = "darkblue", alpha = 0.6) +
  geom_col(fill = "chartreuse3", alpha = 0.8) +
  geom_errorbar(aes(ymin = lower_ci, ymax = upper_ci), 
                colour = "grey30", size = 0.4, width = 8) +
  scale_y_continuous(labels = function(x) paste0(x, "%")) +
  scale_x_date(breaks = bike_booted_rur$recorded_date, date_labels = "%b %y") +
  labs(x = NULL, y = "Median difference in user frequency") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank())
  

urban_plot <- ggplot(bike_booted_urb, aes(x = recorded_date, y = statistic)) +
  geom_hline(yintercept = 0, colour = "darkblue", alpha = 0.8) +
  geom_col(fill = "orange", alpha = 0.8) +
  geom_errorbar(aes(ymin = lower_ci, ymax = upper_ci), 
                colour = "grey30", size = 0.4, width = 8) +
  scale_y_continuous(labels = function(x) paste0(x, "%")) +
  scale_x_date(breaks = bike_booted_rur$recorded_date, date_labels = "%b %y") +
  labs(x = NULL, y = "Median difference in user frequency") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank())


# combine plots -----------------------------------------------------------

# using patchwork library syntax
combined_plot <- summary_plot / rural_plot / urban_plot +
  plot_annotation(tag_levels = "a")

# save plots --------------------------------------------------------------

ggsave(summary_plot, path = here("figures"), filename = "summary_differences.png", dpi = 300)

ggsave(rural_plot, path = here("figures"), filename = "rural_differences.png", dpi = 300)

ggsave(urban_plot, path = here("figures"), filename = "urban_differences.png", dpi = 300)

# combined_plot needs to be saved in proper format
ggsave(combined_plot, filename = here("figures/combined_plot.png"), 
       width = 210, height = 297, units = "mm")
# tables ------------------------------------------------------------------

# use the flextable package to generate nice tables and then export it to a word file using the officer package


# preprocess percentage column
bike_booted_perc <- bike_booted_perc %>% 
  ungroup() %>% 
  select(Monat, type, "percentage" = statistic)

# render flextable
bike_booted_fxt <- bike_booted %>%
  ungroup() %>% 
  # add percentage change column
  left_join(bike_booted_perc) %>% 
  # add date format
  select(month = Monat, everything()) %>% 
  mutate(year = if_else(month %in% c(3:6), 2020, 2019),
         recorded_date = make_date(year = year, month = month),
         recorded_date = format.Date(recorded_date, "%b %y"),
         .before = month) %>% 
  select(-month, -year) %>% 
  # reorder
  select(type, everything()) %>% 
  # round properly
  mutate(across(where(is.numeric), round, 1)) %>% 
  # make a flextable
  flextable() %>% 
  # highlight negative values
  color(i = ~ statistic < 0, j = "statistic", color = "#d84226") %>% 
  color(i = ~ lower_ci < 0, j = "lower_ci", color = "#d84226") %>% 
  color(i = ~ upper_ci < 0, j = "upper_ci", color = "#d84226") %>% 
  color(i = ~ percentage < 0, j = "percentage", color = "#d84226") %>% 
  # lighten all other values
  color(i = ~ statistic >= 0, j = "statistic", color = "grey20") %>% 
  color(i = ~ lower_ci >= 0, j = "lower_ci", color = "grey20") %>% 
  color(i = ~ upper_ci >= 0, j = "upper_ci", color = "grey20") %>% 
  color(i = ~ percentage >= 0, j = "percentage", color = "grey20") %>% 
  # percentage signs
  colformat_int(j = "percentage", suffix = " %") %>% 
  # set column names
  compose(j = "type", part = "header", 
          value = as_paragraph("Sample site")) %>% 
  compose(j = "recorded_date", part = "header", 
          value = as_paragraph("Date")) %>% 
  compose(j = "statistic", part = "header", 
          value = as_paragraph("Median difference")) %>% 
  compose(j = "lower_ci", part = "header", 
          value = as_paragraph("Lower 95% CI")) %>% 
  compose(j = "upper_ci", part = "header", 
          value = as_paragraph("Upper 95% CI")) %>% 
  compose(j = "percentage", part = "header", 
          value = as_paragraph("Percentage change")) %>%
  # bold headers
  bold(part = "header") %>% 
  # italic taxon column
  italic(j = 1) %>%
  # merge sample site 
  merge_v(j = 1) %>%
  # add horizontal line
  hline(i = 8, j = 1:6, border = fp_border(color = "grey10", style = "solid", width = 1)) %>% 
  fix_border_issues() %>% 
  autofit()


# open docx-file and add flextable
my_doc <- read_docx() %>% 
  body_add_flextable(bike_booted_fxt)


# preprocess percentage column
bike_booted_smr_perc <- bike_booted_smr_perc %>% 
  ungroup() %>% 
  select(Jahr, type, "percentage" = statistic)

# same for summaries
bike_booted_smr_fxt <- bike_booted_smr %>% 
  ungroup() %>% 
  # add percentage column
  left_join(bike_booted_smr_perc) %>% 
  # reorder
  select(type, everything()) %>% 
  # round properly
  mutate(across(where(is.numeric), round, 1)) %>% 
  # make a flextable
  flextable() %>% 
  # highlight negative values
  color(i = ~ statistic < 0, j = "statistic", color = "#d84226") %>% 
  color(i = ~ lower_ci < 0, j = "lower_ci", color = "#d84226") %>% 
  color(i = ~ upper_ci < 0, j = "upper_ci", color = "#d84226") %>% 
  color(i = ~ percentage < 0, j = "percentage", color = "#d84226") %>% 
  # lighten all other values
  color(i = ~ statistic >= 0, j = "statistic", color = "grey20") %>% 
  color(i = ~ lower_ci >= 0, j = "lower_ci", color = "grey20") %>% 
  color(i = ~ upper_ci >= 0, j = "upper_ci", color = "grey20") %>% 
  color(i = ~ percentage >= 0, j = "percentage", color = "grey20") %>% 
  # percentage signs
  colformat_int(j = "percentage", suffix = " %") %>% 
  # set column names
  compose(j = "type", part = "header", 
          value = as_paragraph("Sample site")) %>% 
  compose(j = "Jahr", part = "header", 
          value = as_paragraph("Year")) %>% 
  compose(j = "statistic", part = "header", 
          value = as_paragraph("Median difference")) %>% 
  compose(j = "lower_ci", part = "header", 
          value = as_paragraph("Lower 95% CI")) %>% 
  compose(j = "upper_ci", part = "header", 
          value = as_paragraph("Upper 95% CI")) %>% 
  compose(j = "percentage", part = "header", 
          value = as_paragraph("Percentage change")) %>% 
  # bold headers
  bold(part = "header") %>% 
  # italic taxon column
  italic(j = 1) %>%
  # merge sample site 
  merge_v(j = 1) %>%
  # add horizontal line
  hline(i = 2, j = 1:6, border = fp_border(color = "grey10", style = "solid", width = 1)) %>% 
  fix_border_issues() %>% 
  autofit()

# add to word file
my_doc <- my_doc %>% 
  body_add_break() %>% 
  body_add_flextable(bike_booted_smr_fxt, pos = "after")

# render word file
# convert to word file/ add input to empty docx
print(my_doc, target = here("figures/extended_tables.docx"))

